%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  5.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';
   
% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = 1.0       ;   %  High reflectivity on the left facet -- 100 %
BC(2,2) = sqrt(0.05);   %  Low reflectivity on the right facet --   5 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                        %  even when all coefficients are real
                     
%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

% Plot the gain and index to check the correctness of the discretization.
 
figure(1) ; [g_max , om_max , dg_dD , alpha ] = gnr_chi_plot( 3 , M , dt , par(5:9) ) 


%% ------------------3-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past and N 
% points in space. All variables are set to zero excepted the carrier that 
% are set around twice transparency. We add a gaussian fluctuation of 1e-01. 

MSH0 = gnr_uni_msh( M , N , 1.e-01 , 3. )  ;

% plot the profile that corresponds to the initial condition

gnr_msh_plot( MSH0 , 2 , 'pt' ) ;

%% ------------------4-Time integration -----------------------------------

% Time integration over 500 single trips, of one section, saving data
% every 4*dt (Samp=4) and not using decimation (D=1)

S1 = 500*N ; V = 1 ; Samp = 4 ; D = 1 ;

tic; [ tr1 , MSH1 ] = freetwm( S1 , V , Samp , MSH0 , PAR , D ,  BC ) ; toc

gnr_msh_plot( MSH1 , 3 , 'pt' ); % plot the obtained profile, strongly multimode

% Time integration over 3500 single trips, of one section, saving data every 4*dt

S2 = 3500*N ;
tic; [ tr2 , MSH2 ] = freetwm( S2 , V , Samp , MSH1 , PAR , D , BC ) ; toc

gnr_msh_plot( MSH2 , 4 , 'pt' ) ; % plot the final profile, quite monomode

%% ------------------5-Data processing-------------------------------------

tr = [ tr1 ; tr2 ];                      % concatenate the two time traces 
time = ( 1:Samp:(S1+S2) )*(dt)/1.e-09;   % create a time index in nanosecond

Em0 = tr(:,1) ;                  % The output field on the left E_-(0)
EpL = tr(:,2) ;                  % The output field on the right  E_+(1)

D00 = tr(:,3) ;                  % The carrier density on the left  D_0(0)
D0L = tr(:,4) ;                  % The carrier density on the right D_0(1)


fig=figure(5);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot left  output intensity ------------- %

subplot(2,6,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([0 50 0 50]);

subplot(2,6,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([40 40.09 0 20]);

% ------------ Plot right output intensity ------------- %

subplot(2,6,2) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ;  axis([0 50 0 200]);

subplot(2,6,4) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ;  axis([40 40.09 0 100]);
 
% ------------ Plot left  carrier density  ------------- %

subplot(2,6,7) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([0 50 2 5]);

subplot(2,6,9) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([40 40.09 3.43 3.46]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,6,8) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([0 50 2 5]);

subplot(2,6,10) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([40 40.09 2.58 2.65]);


% We perform a Optical spectrum of the field once it reached steady state : we keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL(end/2:end)     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL(end/2:end)).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,6,[5 6]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([1.25 2.75]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,6,[11 12]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %


