%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';
             
% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %
BC(2,2) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %

BC = complex(BC) ;      %  The BC matrix must be complex                           

%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

%% ------------------3-Etching the saturable absorber section ------------- 

% The absorber is 3% of the cavity, i.e. 400*0.03=12 pts, and located on the right.
% We assume that the absorption of the absorber is x 3 the one of the gain section. 
% We move the absorber band-edge of 5 THz on the blue, (closer to the gain peak.)
% The recovery time is 15 ps.
 
NSA = N-12+1:N ;

PAR( 4,NSA) = 3 * PAR(4,NSA) ;  % Relative redefinition of a parameter (here the gain)
PAR( 6,NSA) = 5e12*dt        ;  % Absolute redefinition of a parameter (here the bandgap)
PAR(11,NSA) = 0.             ;  % No bias current in the absorber
PAR(13,NSA) = 0.             ;  % No bilinear recombination
PAR(14,NSA) = 0.             ;  % No Auger recombination
PAR(12,NSA) = (1./15e-12)*dt ;  % 15 ps saturable absorber linear recovery time 


%% ------------------4-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 2.5 transparency. We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-01 , 3. )  ;

% The absorber section should be empty of carriers. Therefore 

MSH0(:,NSA,5) = 0 ;

% plot the profile that corresponds to the initial condition

gnr_msh_plot( MSH0 , 1 , 'pt' );

%% ------------------5-Time integration -----------------------------------

% Time integration, over 100 single trips, of one section, 
% saving data every dt (Samp=1) not using decimation (D=1)

S = 1000*N ; V = 1 ; Samp = 1 ; D = 1;

tic; [ tr1 , MSH1 ] = freetwm( S , V , Samp , MSH0 , PAR , D , BC ) ; toc

gnr_msh_plot( MSH1 , 2 , 'pt' ); % plot the obtained profile, pulsating


%% ------------------6-Data processing-------------------------------------

time = (1:S)*(dt)/1.e-09;    % create a time index in nanosecond

Em0 = tr1(:,1) ;                  % The output field on the left E_-(0)
EpL = tr1(:,2) ;                  % The output field on the right  E_+(1)

D00 = tr1(:,3) ;                  % The carrier density on the left  D_0(0)
D0L = tr1(:,4) ;                  % The carrier density on the right D_0(1)

fig=figure(3);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot left  output intensity ------------- %

subplot(2,6,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([0 12.5 0 500]);

subplot(2,6,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([12 12.09 0 500]);

% ------------ Plot right output intensity ------------- %

subplot(2,6,2) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ;  axis([0 12.5 0 500]);

subplot(2,6,4) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ;  axis([12 12.09 0 500]);
 
% ------------ Plot left  carrier density  ------------- %

subplot(2,6,7) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([0 12.5 2 4]);

subplot(2,6,9) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([12 12.09 2 4]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,6,8) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([0 12.5 0 1]);

subplot(2,6,10) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([12 12.09 0 1]);


% We perform a Optical spectrum of the field once 
% it reached steady state and keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL(end/2:end)     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL(end/2:end)).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,6,[5 6]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([0 2]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,6,[11 12]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %






