%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30)  ;  %  Cleaved facet reflectivity -- 30 %
BC(2,2) =      0.0    ;  %  AR facet on the right      --  0 %

BC = complex(BC) ;  %  The BC matrix must be complex 
                      
%   [g_max , om_max , dg_dD , alpha ] = gnr_chi_plot( 3 , M , dt , par(5:9) )      

%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

%% ------------------3-Etching the Distributed Bragg Grating  ------------- 

% The grating is 25% of the cavity, i.e. 400*0.25=100 pts. We created a 
% index range for the grating that is located on the right and assume a
% purely passive section, i.e. all parameters are set to zero

NDBR = N-100+1:N ;  
PAR( :,NDBR) = 0 ; 

% We assume that  kappa0*L_dbr = 0.62 such that the amplitude reflectivity 
% of the grating is tanh(0.62) = 0.55, i.e. like an as cleaved facet.

kappa0 = 0.62/100         ;              % L_dbr 0.25*L = 0.25*(400) = 100 


h = 1./N                  ;              % Spatial increment
z = linspace(h/2,1-h/2,N) ;              % Normalized index along the cavity
x =  z(NDBR)              ;              % We redefine an origin at the DBR start

Beta_Bragg = 10*(pi);                    % 10 modal separation of a Fabry-Pérot laser

kappa = kappa0*exp( 2i*Beta_Bragg*x ) ;  % Harmonic variation of kappa @ Beta_Bragg

PAR(2,NDBR) = real(kappa) ;              % Introduce the grating in the parameter
PAR(3,NDBR) = imag(kappa) ;              % matrix, real and imaginary parts


% A nice plot the real and imaginary part of the distributed feedback coupling 

fig=figure(1); clf; 
set( fig , 'Position' , [0 0 1200 750] ) ;
set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );

plot(z,PAR(2,:),'o--' ,z, PAR(3,:),'--o', 'LineWidth', 2);    
grid on ;xlabel('z/L (a.u.)') ; axis tight; xlim([0 1]);
legend('\Re( \kappa )','\Im( \kappa )','Location','NorthWest');


%% ------------------4-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 2.5 transparency. We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-03 , 3. )  ;

% The grating section should be empty of population inversion, therefore 

MSH0(:,NDBR,5:6) = 0 ; %MSH0=complex(MSH0);

%% ------------------5- Transient regime toward steady state --------------

% Time integration, over 500 single trips, of one section, saving data every dt

S = 500*N ; V = 1 ; Samp = 1 ; D = 1;

tic; [ diag(1).tr , diag(1).msh ] = freetwm( S , V , Samp , MSH0 , PAR , D , BC ) ; toc

gnr_msh_plot( diag(1).msh , 2 ) ; % Plot the obtained spatial profiles 

[nu ,diag(1).spec ] = OpticalSpectrum( diag(1).tr((end/2):end,1) , dt/1e-12 , 'normalize' );


%% ------------------6-Data processing-------------------------------------

fig=figure(3); clf; set( fig , 'Position' , [0 0 1200 750] ) ;

subplot(2,2,[1 3]); set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );

tns=(1:S)*Samp*dt/1e-09 ;  % time index in ns

plot(tns,abs(diag(1).tr(:,2)).^2,'r', 'LineWidth', 2); 
grid on ; axis tight ; xlabel('Time (ns)') ; legend('I_+(1)') ; 

subplot(2,2,2); set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );
semilogy(nu,abs(diag(1).spec).^2, 'LineWidth', 2 ) ;
xlim([-15 15]) ; ylim([1.e-10 1]); grid on; 
xlabel('Frequency (THz)') ; legend('OS (dB)') ; 

subplot(2,2,4); set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );
semilogy(nu,abs(diag(1).spec).^2, 'LineWidth', 2 ) ;
xlim([0.1 0.7]); ylim([1.e-10 1]); grid on; 
xlabel('Frequency (THz)') ;  legend('OS (dB)') ; 


%% ------------------7-Time integration & Tuning of the phase  ------------
% -------Beware that this block is 20 time longer than section 5 !! -------

tic;
for ii=2:20,
    
Beta_Bragg = (9+ii)*(pi)              ;  % Tuning of  Beta_Bragg in FP resonance units
kappa = kappa0*exp( 2i*Beta_Bragg*x ) ;  % Harmonic variation of kappa @ Beta_Bragg
PAR(2,NDBR) = real(kappa) ;  
PAR(3,NDBR) = imag(kappa) ;            


[ diag(ii).tr , diag(ii).msh ] = freetwm( S , V , Samp , diag(ii-1).msh , PAR , D , BC ) ; 

[nu , diag(ii).spec ] = OpticalSpectrum( diag(ii).tr(130000:end,1) , dt/1.e-12 , 'normalize' );

end
toc;

%% ------------------8-Data processing-------------------------------------

Col=jet(20); Col=Col([20:-1:1],:); % blue to red colors

K1=2; K2=20;

fig=figure(4); clf; set( fig , 'Position' , [0 0 1200 750] ) ;

subplot(1,2,1); set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );

for ii=K1:K2, 
    plot(tns,abs(diag(ii).tr(:,2)).^2+(ii-1)*10,'Color',Col(ii,:), 'LineWidth' , 2 ) ; 
    hold on ; 
end

axis tight; grid on; xlabel('Time (ns)') ; legend('I_+(1)') ; 

subplot(1,2,2); set( gca , 'FontSize' , 22 , 'LineWidth' , 2 );

for ii=K1:K2 , 
    semilogy( nu , abs(diag(ii).spec).^2 , 'Color' ,Col(ii,:),'LineWidth',2) ; 
    hold on;
end

axis([0.25 1.25 1.e-10 1]);grid on; xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 








