%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;   % cavity single trip (i.e. 80 GHz rep rate for a ring)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  5.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,2) = sqrt(0.50);  % 50% Transmission of the coupler 
BC(2,1) = sqrt(0.50);  % 50% Transmission of the coupler 

BC(1,1) = 1.e-03 ;   %  Reflectivity of A-(0) into A+(0) 
BC(2,2) = 1.e-03 ;   %  Reflectivity of A+(0) into A-(0) 

BC = complex(BC) ;      %  The BC matrix must be complex 
                        %  even when all coefficients are real

                     
%% ------------------2-Discretization and scaled parameter profiles--------

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

% We do not plot the gain and index to check the correctness of the discretization.
% as the parameters here are identical to the one in the first example.

%% ------------------3-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carriers 
% that are set around 2.5 transparency. We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-03 , 3.0 )  ;

%% ------------------4-Injection Signal------------------------------------ 

% We assume that we are going to make a time integration over 500 single trips. 

S1 = 500*N ; 

% Therefore, we must prepare the injected fields over this time span.
% We start by creating a time index ( in ps ). Also, we preallocate a two 
% dimensional Matrix for the left & right injection with the correct size.

t = (1:S1)*dt ; Inj = zeros( S1 , 2 ) ; 

% We will inject a monochromatic field. Into the forward direction. 
% Doing so, we enforce clockwise operation in this bistable system.
% We need to define a injection field with a rising and a falling edge.
% Otherwise the unphysical field discontinuity triggers multimode dynamics.

rise_edge = 0.5*( 1 + tanh( (t- 200.e-12)/50e-12) );  % 50 ps rise, 200 ps offset
fall_edge = 0.5*( 1 + tanh(-(t-1100.e-12)/20e-12) );  % 20 ps fall, duration ~ 900 ps

% The carrier frequency of the injection is actually the one of the free 
% lasing operation and upon releasing injection (after the falling edge) 
% the emission will persists at the same frequency. 

NU_laser  = 1.746e12;                          % NU > 0 => blue wrt band gap.
carrier   = exp(-2i*pi*NU_laser*t);             % We are in --minus-- representation

% At last, we choose the amplitude

amp       = sqrt(20.);                          % sqrt(Intensity)

% This defines the injection fields on both inputs

Inj(:,1) = amp*carrier.*rise_edge.*fall_edge ;  % We inject in E+(0) 
Inj(:,2) = 0.                                ;  %  and not in  E-(1)


%% ------------------5-Transient turn on-----------------------------------

% We have one section and we save the output every point 

 V = 1 ; Samp = 1 ; D =  1 ;    
 
 % First, we inject during the turn on transient, to ensure Clockwise operation
 
tic; [ tr1 , MSH1 ] = freetwm( S1 , V , Samp , MSH0 , PAR , D , BC , Inj ) ; toc

% plot the obtained time trace, Clockwise emission

fig=figure(1);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

subplot(2,2,1); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,real(Inj(:,1)),'k',t/1.e-09,abs(Inj(:,1)),'or', 'LineWidth', 2);
legend('\Re(Y_+)','|Y_+|'); xlabel('time (ns)'); axis tight ; grid on ;

subplot(2,2,3); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,abs(tr1(:,2)).^2,'--r',t/1.e-09,abs(tr1(:,1)).^2,'b', 'LineWidth', 2) ; 
legend('I_+','I_-'); xlabel('time (ns)'); axis tight ; ylim([0  150]); grid on ;


%% -------------------6-Direction reversal by a broad pulse ---------------

% Now, we are going to inject in the other direction in order to reverse
% the direction of emission. Instead of creating an other injection field, 
% an easy way is to reuse the previous one. We do so by swapping the two 
% columns of the previous injection matrix, i.e. 

Inj(:,[2 1]) = Inj(:,[1 2]) ; 

tic; [ tr2 , MSH2 ] = freetwm( S1 , V , Samp , MSH1 , PAR , D , BC , Inj ) ; toc

% plot the obtained time trace, counter clockwise emission

subplot(2,2,2); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,real(Inj(:,2)),'--k',t/1.e-09,abs(Inj(:,2)),'ob', 'LineWidth', 2);
legend('\Re(Y_-)','|Y_-|'); xlabel('time (ns)'); axis tight ; grid on ;

subplot(2,2,4); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,abs(tr2(:,2)).^2,'--r',t/1.e-09,abs(tr2(:,1)).^2,'b', 'LineWidth', 2) ; 
legend('I_+','I_-'); xlabel('time (ns)'); axis tight ; ylim([0  150]); grid on ;



%% -------------------7-Detuned injection ---------------------------------

% We inject again in the forward direction, but 20 modes to the red, i.e. NU~0.

carrier   = exp(-2i*pi*(0)*t);                  % New carrier frequency

Inj(:,1)=amp*carrier.*rise_edge.*fall_edge   ;  % We inject in E+(0) 
Inj(:,2)= 0.                                 ;  %  and not in  E-(1)

Inj=complex(Inj);       %  The Inj field matrix must be complex 
                        %  even when all coefficients are real

tic; [ tr3 , MSH3 ] = freetwm( S1 , V , Samp , MSH2 , PAR , D , BC , Inj ) ; toc


% plot the obtained time trace, no reversal and still counter clockwise emission

fig=figure(2);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

subplot(2,2,1); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,real(Inj(:,1)),'--k',t/1.e-09,abs(Inj(:,1)),'or', 'LineWidth', 2);
legend('\Re(Y_+)','|Y_+|'); xlabel('time (ns)'); axis tight ; grid on ;

subplot(2,2,3); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,abs(tr3(:,2)).^2,'--r',t/1.e-09,abs(tr3(:,1)).^2,'b', 'LineWidth', 2) ; 
legend('I_+','I_-'); xlabel('time (ns)'); axis tight ; ylim([0  150]); grid on ;

%% -------------------8-Direction reversal by a short pulse ---------------

% At last we inject a pulse whose spectrum is larger than the modal
% separation. This will trigger a multimode directional reversal.

NU_laser  = 1.746e12;                                         % NU > 0 => blue wrt band gap.
carrier   = exp(-2i*pi*NU_laser*t);                           % We are in --minus-- representation
amp       = sqrt(200.);                                       % sqrt(peak power)
pulse     = exp( - (t - 200.e-12).^2 / (0.7213*(5e-12)^2) ) ; % 5 ps FWHM ; 200 ps offset

Inj(:,1) =  amp*pulse.*carrier;
Inj(:,2) =  0. ;

tic; [ tr4 , MSH4 ] = freetwm( S1 , V , Samp , MSH2 , PAR , D , BC , Inj ) ; toc

% plot the obtained time trace, multimode clockwise emission

subplot(2,2,2); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,real(Inj(:,1)),'--k',t/1.e-09,abs(Inj(:,1)),'or', 'LineWidth', 2);
legend('\Re(Y_+)','|Y_+|'); xlabel('time (ns)'); axis tight ;xlim([0.18  0.22]); grid on ;

subplot(2,2,4); set(gca,'FontSize',22,'LineWidth',2);
plot(t/1.e-09,abs(tr4(:,2)).^2,'--r',t/1.e-09,abs(tr4(:,1)).^2,'b', 'LineWidth', 2) ; 
legend('I_+','I_-'); xlabel('time (ns)'); axis tight ; ylim([0  250]); grid on ;


