%% ------------------0-Add functions to the search path--------------------
addpath(path,'../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   (unsused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3.e-03  ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';

% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %
BC(2,2) =      0.    ;  %       AR facet reflectivity --  0 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                               
%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

% We do not plot the gain and index to check the correctness of the discretization.
% as the parameters here are identical to the one in the first example.

%% ------------------4-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 2.5 transparency. We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-03 , 3.0 )  ;

%% ------------------5-Time integration -----------------------------------

% Time integration of one section, 3000 single trips, i.e. S=3000*N steps, 
% with delayed boundary conditions. To use the external delayed 
% Boundary conditions, we MUST save data every dt.

Samp = 1 ; V = 1 ; S = 3000*N ; D = 1 ;

% The passive external section is 10.5 cm and the right reflectivity is 0.55.
% The passive external section corresponds to 100 single trips in the amplifier,
% that is to say K=100*N=40000 pts. Hence, we do K integration steps, S/K=30 times,

R_e = 0.55;   K = 100*N ; SoK=S/K;

trf = zeros(S,6)   ;   % preallocate the full output time trace (for speed)
Inj = zeros(K,2)   ;   % Injection fields, initialy zero
Inj = complex(Inj) ;   % it has to be complex 

tic;

for ii=1:SoK,
    
[ tr , MSH0 ] = freetwm( K , V , Samp , MSH0 , PAR , D , BC ,  Inj ) ;

Inj(:,2) =  R_e*tr(:,2) ;  % Delayed Boundary conditions-- Y_-(t)= R*Ep(t-tau); (first one in tr is the oldest).

tr1( [ 1+(ii-1)*K:K*(ii) ] , : ) = tr ;  % append the new chunk of saved output data to the full trace

end
toc

gnr_msh_plot( MSH0 , 3 , 'pt' ); % plot the obtained profile, pulsating


%% ------------------6-Data processing-------------------------------------

time = [1:S]*(dt)/1.e-09;    % create a time index in nanosecond

EpL = tr1(:,2)    ;          % The output     field on the right  E_+(1)
IpL = abs(EpL).^2 ;          % The output intensity on the right  I_+(1)

fig = figure(4) ; clf ; set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot right output intensity ------------- %

subplot(2,2,[1 3]) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , IpL , 'r-' ,time , smooth(smooth(IpL,10*N),10*N) , 'bo' ) ;

legend(' I_+(1) ','<I_+(1)>','Location','Best') ; 
xlabel('Time (ns)') ;  grid on; axis tight; 

% it reached steady state and keep only half of the points.

[ nuthz , specEp ] = OpticalSpectrum( EpL(3*end/4:end) , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nughz , specIp ] = OpticalSpectrum( IpL(3*end/4:end) , Samp*dt*1.e09 , 'normalize' ) ;  % step in pns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,2,2) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nuthz,abs(specEp).^2) ; 
xlim([-1 3]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,2,4) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nughz,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %






