%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 14 parameters (in s^-1)
      
par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   ( unused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3e-03   ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';     

% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30);   %  Cleaved reflectivity on the left facet -- 30 %
BC(2,2) = sqrt(0.30);   %  Cleaved reflectivity on the right facet -- 30 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                        %  even when all coefficients are real
                     
% ------------------2-Discretization and scaled parameter profiles--------

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

% ------------------3-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 1.5 transparency. We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-03 , 2.45 )  ;


%% ------------------3-  feature insertion --------------------------------

pos = [ 200 250 300 350 ];

PAR(2,pos) = 0.1;

fig=figure(1);clf;
set( fig , 'Position' , [0 0 1200 750] ) ;
set(gca,'FontSize',22,'LineWidth',2);

hold on; plot(PAR(2,:),'--ob','LineWidth',2); plot(PAR(3,:),'--og','LineWidth',2); 
legend('\Re(\kappa)','\Im(\kappa)');

%% ------------------4-Time integration -----------------------------------

% Time integration, over 500 single trips, of one section, saving data every 2*dt.

S1 = 1000*N ; V = 1 ; Samp = 1 ; D = 1 ;

tic; [ tr , MSH1 ] = freetwm( S1 , V , Samp , MSH0 , PAR , D , BC ) ; toc

gnr_msh_plot( MSH1 , 2 , 'pt' ); % plot the obtained profile, strongly multimode


%% ------------------5-Data processing-------------------------------------

time = (1:S1)*dt/1.e-09;        % create a time index in nanosecond

EpL = tr(:,2) ;                  % The output field on the right  E_+(1)
D0L = tr(:,4) ;                  % The carrier density on the right D_0(1)

fig=figure(3);clf;set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot right output intensity ------------- %

subplot(2,2,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;xlabel('Time (ns)') ; 
grid on ;  axis([11 11.1 0 75]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,2,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([11 11.1 2.9 2.92]);

% We perform a Optical spectrum of the field once 
% it reached steady state and keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL(end/2:end)     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL(end/2:end)).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,2,2) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([0 2]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,2,4) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 500]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %


