%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters------------------------------------------

tau_c = 12.5e-12 ;              % cavity single trip (i.e. 40 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 3.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  8.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  5.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   ( unused)
 150e08   ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e08   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  7.e08   ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  1.e07   ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3e-03   ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';     

% The boundary matrix must link the two outputs Em(0) and Ep(L) 
% to the two inputs that are Ep(0) and Em(L), hence it is a 2x2 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved reflectivity  -- 30 %
BC(2,2) = sqrt(0.30) ;  %  Cleaved reflectivity  -- 30 %

BC = complex(BC) ;      %  The BC matrix must be complex 
                        %  even when all coefficients are real
                     
%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space.                        
% M  ; Number of point in the past.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;
 
PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N pts

% Plot the gain and index to check the correctness of the discretization.
 
% figure(1) ; [g_max , om_max , dg_dD , alpha ] = gnr_chi_plot( 3 , M , dt , par(5:9) ) 


%% ------------------3-Spatially uniform IC--------------------------------

% Generate an initial condition, i.e. a mesh of M points in the past and N 
% points in space. All variables are set to zero excepted the carrier that 
% are set around twice transparency. We add a gaussian fluctuation of 1e-01. 

MSH0 = gnr_uni_msh( M , N , 1.e-01 , 3. )  ;

% plot the profile that corresponds to the initial condition

% gnr_msh_plot( MSH0 , 2 , 'pt' ) ;


%% ------------------4-Modulation Signals --------------------------------- 

% We assume that we are going to make a time integration over 800 single trips. 

S = 800*N ; 

% Therefore, we must prepare the modulation signal over this time span.
% We start by creating a time index ( in ps ). Also, we preallocate a two 
% dimensional Matrix for the modulation of the parameters with the correct size.

t = ( (1:S) - 1 )*dt ; Mod = zeros( S , 3 ) ; 

% First, we will modulate the bias current harmonically at a frequency of 5 GHz

NU_mod  = 5e09;       

% At last, we choose the amplitude
% the modulation amplitude is going to be 10% of the steady value
% As all parameters, the current must be scaled by the time step dt

amp = 0.1*par(11)*dt ;

% This defines the modulation matrix

Mod( : , 2 ) = amp*sin(2*pi*NU_mod*t);

% Also, we can make a 100% modulation at 1 GHz in order to trigger a Q-Switched regime

% NU_mod  = 1e09 ; amp = 1*par(11)*dt ; Mod( : , 2 ) = amp*sin(2*pi*NU_mod*t);

%% ------------------5-Time integration -----------------------------------

% Time integration of one section, saving data every 4*dt (Samp=4) without decimation (D=1)

V = 1 ; Samp = 4 ; D = 1 ;

tic; [ tr , MSH1 ] = freetwm( S , V , Samp , MSH0 , PAR , D ,  BC , Mod ) ; toc


%% ------------------5-Data processing-------------------------------------

time = ( (1:Samp:S) - 1)*(dt)/1.e-09;   % create a time index in nanosecond

Em0 = tr(:,1) ;                  % The output field on the left E_-(0)
EpL = tr(:,2) ;                  % The output field on the right  E_+(1)

D00 = tr(:,3) ;                  % The carrier density on the left  D_0(0)
D0L = tr(:,4) ;                  % The carrier density on the right D_0(1)

fig = figure(5) ; clf ; set( fig , 'Position' , [0 0 1600 750] ) ;
set(gca,'LooseInset',get(gca,'TightInset'))

% ------------ Plot left  output intensity ------------- %

subplot(2,6,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis tight ; 

subplot(2,6,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis tight ; xlim([8 10]);

% ------------ Plot right output intensity ------------- %

subplot(2,6,2) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ; axis tight ;  

subplot(2,6,4) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ;%xlabel('Time (ns)') ; 
grid on ; axis tight ; xlim([8 10]);
 
% ------------ Plot left  carrier density  ------------- %

subplot(2,6,7) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis tight ;
subplot(2,6,9) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ; axis tight ; xlim([8 10]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,6,8) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis tight ;

subplot(2,6,10) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis tight ; xlim([8 10]);


% We perform a Optical spectrum of the field once it reached steady state : we keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL(:)     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL(:)).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,6,[5 6]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([1.25 3.75]); 
ylim([1.e-7 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,6,[11 12]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-06 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %


