%% ------------------0-Add functions to the search path--------------------
addpath('../System');

%% ------------------1-Parameters of the two sections----------------------

tau_c = 3.125e-12 ;              % cavity single trip (i.e. 160 GHz rep rate)

% par is a vector containing the values of the 16 parameters (in s^-1)

% parameter for the absorber section %

par = [
 6.0e10   ...   % 1---waveguide  response : internal losses        (s^-1)
  0.e11   ...   % 2---waveguide  response : real part of the dfb   (s^-1)
  0.e11   ...   % 3---waveguide  response : imag part of the dfb   (s^-1)
 9.0e11   ...   % 4----Material  response : sat gain/abs factor    (s^-1)
  2.e12   ...   % 5----Material  response : intraband relax time   (s^-1)
  0.e12   ...   % 6----Material  response : bandgap offset     (rad.s^-1)
 90.e12   ...   % 7----Material  response : Span of the band   (rad.s^-1)
  8.e11   ...   % 8----Material  response : elec thermal width (rad.s^-1)
  8.e12   ...   % 9----Material  response : hole thermal width (rad.s^-1)
  0.      ...   % 10---Material  response : intraband sat nrj   ( unused)
  0.      ...   % 11----Carrier  response : injected carrier rate  (s^-1)
  1.e11   ...   % 12----Carrier  response : non rad decay rate     (s^-1)
  0.      ...   % 13----Carrier  response :     rad decay rate     (s^-1)
  0.      ...   % 14----Carrier  response :   Auger decay rate     (s^-1)
  1.e12   ...   % 15----Carrier  response :  Pop Grating diff rate (s^-1)  
  3e-03   ...   % 16----Carrier  response : Sp emission rate        (a.u)                               
 ]';     

% The boundary matrix must link the two outputs Em(0,1:2) and Ep(L,1:2) of 
% the two sections to the two inputs of the two sections Ep(0,1:2) and Em(L,1:2)
% Hence the boundary matrix is a 4x4 matrix 

BC = zeros(2,2)  ; 

BC(1,1) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 %
BC(2,2) = sqrt(0.30) ;  %  Cleaved facet reflectivity -- 30 % 

BC = complex(BC) ;      %  The BC matrix must be complex 
                                           
%% ------------------2-Discretization and scaled parameter profiles--------                          

% N  ; Number of point in space    for the first (second) section.                       
% M  ; Number of point in the past for the first (second) section.
% dt ; Time step used in the parameter scaling

[ M , N , dt ] = gnr_good_discr( tau_c , par ) ;

PAR   = gnr_uni_param( N , dt , par ) ; % An uniform scaled parameter set of N1 pts

figure(1) ; clf ; gnr_chi_plot( 0.01 , M , dt , par(5:9) )  ; % the saturable aborption

%% ------------------3-Spatially uniform IC for the two sections-----------

% Generate an initial condition, i.e. a mesh of M points in the past
% and N points in space. All variables are set to zero excepted the carrier 
% that are set around 2.5 in the gain section and 0 in the absorber. 
% We add a gaussian fluctuation of 1.e-03. 

MSH0 = gnr_uni_msh( M , N , 1.e-01 , 0.01 )  ;

%% ------------------4-Modulation Signals --------------------------------- 

% We assume that we are going to make a time integration over 800 single trips. 

S = 1000*N ;

% Therefore, we must prepare the modulation signal over this time span.
% We start by creating a time index ( in ps ). Also, we preallocate a two 
% dimensional Matrix for the modulation of the parameters with the correct size.

t = ( (1:S) - 1 )*dt ; Mod = zeros( S , 3 ) ; 

% We will modulate the Voltage, hence the bandgap harmonically at 5 GHz

NU_mod  = 5.e09;       

% At last, we choose the amplitude of the modulation as 1 THz.
% As all parameters, this modulation must be scaled by the time step dt

amp = (2*pi)*(0.5e12)*(dt) ; % 2pi because the gap is in Rad.THz 

% This defines the modulation matrix, where we modulate the first
% parameter among the three possible choices, i.e. the bandgap

Mod( : , 1 ) = amp*sin(2*pi*NU_mod*t);

%% ------------------5-Injection Signal------------------------------------ 

% Therefore, we must prepare the injected fields. Also, we preallocate a two 
% dimensional Matrix for the left & right injection with the correct size.

Inj = zeros( S , 2 ) ; 

% We will inject a monochromatic field from the left facet
% We need to define a injection field with a rising and a falling edge.
% Otherwise the unphysical field discontinuity triggers multimode dynamics.

rise_edge = 0.5*( 1 + tanh( (t- 500.e-12)/100e-12) );  % 10 ps rise, 100 ps offset
fall_edge = 0.5*( 1 + tanh(-(t-2500.e-12)/100e-12) );  % 10 ps fall, duration ~ 3 ns

% The carrier frequency of the injection is actually the one of the band edged

NU_laser  = 0.;                          % NU > 0 => blue wrt band gap.
carrier   = exp(-2i*pi*NU_laser*t);      % We are in --minus-- representation

% At last, we choose the amplitude

amp       = sqrt(20.);                   % sqrt(Intensity)

% This defines the injection fields on both inputs

Inj(:,1) = amp*carrier.*rise_edge.*fall_edge ;  % We inject in E+(0) 
Inj(:,2) = 0.                                ;  %  and not in  E-(1)

Inj = complex( Inj ) ;      %  The Injection matrix must be complex 

%% -----------------6-Time integration -----------------------------------

% Time integration, over 100 single trips, of two sections (V=2), 
% saving data every dt (Samp=1) not using decimation for both sections (D1=D2=1)

 V = 1 ; Samp = 1 ; D = 1 ; 

tic; [ tr1 , MSH1 ] = freetwm( S , V , Samp , MSH0 , PAR , D ,  BC , Inj , Mod ) ; toc

%% ------------------6-Data processing-------------------------------------

time = (1:S)*(Samp*dt)/1.e-09;   % create a time index in nanosecond

Em0 = tr1(:,1) ;                  % The output field on the left  E_-(0)
EpL = tr1(:,2) ;                  % The output field on the right E_+(1)

D00 = tr1(:,3) ;                  % The carrier density on the left  D_0(0)
D0L = tr1(:,4) ;                  % The carrier density on the right D_0(1)

fig=figure();clf;set( fig , 'Position' , [0 0 1600 750] ) ;

% ------------ Plot left  output intensity ------------- %

subplot(2,6,1) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([0 3 0 0.6]);

subplot(2,6,3) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(Em0).^2 , 'b' , 'LineWidth', 2) ;    
legend('I_-(0)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on; axis([1 1.5 0 0.6]);

% ------------ Plot right output intensity ------------- %

subplot(2,6,2) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on ; axis([0 3 0 8]);

subplot(2,6,4) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time , abs(EpL).^2 , 'r' , 'LineWidth', 2) ;  
legend('I_+(1)','Location','Best') ; %xlabel('Time (ns)') ; 
grid on ; axis([1. 1.5 0 8]);

% ------------ Plot left  carrier density  ------------- %

subplot(2,6,7) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ; axis([0 3 0 0.4]);

subplot(2,6,9) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D00     , 'k' , 'LineWidth', 2) ;  
legend('D_0(0)','Location','Best') ;  xlabel('Time (ns)') ; 
grid on ;  axis([1 1.5 0 0.4]);

% ------------ Plot right  carrier density  ------------- %

subplot(2,6,8) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([0 3 0 0.05]);

subplot(2,6,10) ; set(gca,'FontSize',22,'LineWidth',2);
plot ( time ,     D0L     , 'k' , 'LineWidth', 2) ;  
legend('D_0(1)','Location','Best') ; xlabel('Time (ns)') ; 
grid on ; axis([1 1.5 0 0.05]);


% We perform a Optical spectrum of the field once 
% it reached steady state and keep only half of the points.

[ nu1 , specEp ] = OpticalSpectrum(     EpL     , Samp*dt*1.e12 , 'normalize' ) ;  % step in ps
[ nu2 , specIp ] = OpticalSpectrum( abs(EpL).^2 , Samp*dt*1.e09 , 'normalize' ) ;  % step in ns

% ------------ Plot the  optical spectrum ------------- %

subplot(2,6,[5 6]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu1,abs(specEp).^2) ; 
xlim([-0.2 0.2]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (THz)') ; 
legend('OS (dB)','Location','SouthEast') ; 

% ------------ Plot the RF spectrum  ------------------ %

subplot(2,6,[11 12]) ;  set(gca,'FontSize',22,'LineWidth',2);
semilogy(nu2,abs(specIp).^2, 'LineWidth', 2) ;   
xlim([0 50]); 
ylim([1.e-10 1]); 
grid on;
xlabel('Frequency (GHz)') ; 
legend('RF (dB)','Location','Best');

%------------------------------------------------------ %







