function [nu , spec , spec_mean , spec_fwhm , mode_amp , freq_id ] = OpticalSpectrum( E , dt , flag , mode_sep )

% [nu , spec , spec_mean , spec_fwhm , mode_amp , freq_idx ] = OpticalSpectrum( E , dt , flag , mode_sep )
% 
% Perform the optical spectrum of a complex field E(t) via the fft method.
% The fft convention is inverse (i.e. we use ifft). A Hann window is used.
% 
%         E: complex vector of N elements (even). If N is odd, we use the closest smaller even value 
%    
%        dt: Sampling rate between points
%    
%      flag: if flag='normalize' normalize the spectrum to 0dB (Optional).
%    
%  mode_sep: approximate modal separation for mean and fhwm estimation (Optional).
%    
% 
%    
%        nu: Unwarped frequency axis from - 1/(2*dt) to + 1/(2*dt).
%
%      spec: Unwarped complex Fourier transform of E.
%
% spec_mean: central frequency of the power spectrum (|spec|^2). (Optional)
%
% spec_fwhm: full width at half maximum of the power spectrum (|spec|^2). (Optional)
%
%  mode_amp: Reconstructed temporal evolution of the complex enveloppe of the modes (Optional)
%
% freq_idx : index of the mode optical frequencies: more are ordered by decreasing power from 0 to -20dB (Optional)


 N = numel(E);

 evenPTS= 2 * floor( N/2 )    ;

 E = E(1:evenPTS); 
 
 if ( nargout < 5 ), hann_ = sin( linspace(0,pi,evenPTS)' ) .^2 ; else hann_ = 1 ; end %no windowing if modal reconstruction
  
 spec = fftshift( ifft( E(:).*hann_(:) ) ) ;
 
 nu  = 1:evenPTS ; nu  = nu' - evenPTS/2 - 1 ; nu = nu / (evenPTS*dt) ;
 
if( nargin>2 && strcmp(flag,'normalize')==1),

 maxspec =  max( abs(spec) ) ;  % Normalize the spectrum such that
   spec  = spec ./ (maxspec) ;  % the maximum is at(end/2:end 0 dB.
end


if ( nargin == 4) %nargout >= 4 &&
    
    dnu   = nu(2)-nu(1);
    
    % find the modes %
    
    bob    = abs(spec).^2;
    bob    = bob/max(abs(bob)) ; % Normalize the temp power spectrum to 0 dB.
    window = floor( mode_sep/dnu / 2 ) ;
    
    Y = 1; I=1;
    
    mode_found = 0 ;
    
    while( Y > 1.e-2  ) % 20 dB cut-off
        
        mode_found = mode_found + 1 ;
        [ Y(mode_found) , I(mode_found) ] = max(bob) ; % get position and value
        
        index_start = I(mode_found) - window  ;
        index_end   = I(mode_found) + window  ;
        
        if(index_end >=evenPTS || index_start <= 1 ) break; end
        
        bob( index_start:index_end ) = 0 ;          % delete from the array
        
               if ( nargout == 6 )
                    tmp_spec=zeros(evenPTS,1);
                    han_win = sin( linspace(0,pi,2*window+1)' ) .^2 ; % modal hann window
                  
                    tmp_spec([-window:window]+evenPTS/2) = spec( index_start:index_end ).*han_win ; %centered around 0
                   %tmp_spec( index_start:index_end) = spec( index_start:index_end ).*han_win ;
                    mode_amp(:,mode_found) = fft( fftshift(tmp_spec) ) ;
               end
        
    end
    
    disp(['Number of modes found at 20 dB  :  ' num2str(mode_found)]);
    % find the fwhm from the second order cumulant
    % fwhm= 2.35482 * variance ;  2sqrt(2ln(2))=2.35482.
    
    Y=Y';
    spec_mean = sum( nu(I).*Y)/sum(Y);
    spec_fwhm = 2.35482 * sqrt(  sum( (nu(I) - spec_mean).^2 .*Y )/sum(Y)  );
    
    freq_id = nu(I) ; % save the frequency of the reconstructed modes
    
end

end


