/*
----------------------------Authors -----------------------------------

Julien Javaloyes
Salvador Balle

Universitat de les Illes Baleares
Palma de Mallorca

-----------------------------Copyright --------------------------------

This file is part of FreeTwm

FreeTwm is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

FreeTwm is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Freetwm; see the file COPYING.  
If not, see <http://www.gnu.org/licenses/>.

-----------------------------------------------------------------------
*/

#include <mex.h>
#include <math.h>
#include <complex.h>
#include <stdio.h>  
#include <stdlib.h>
#include <string.h>
/*#include <omp.h> */                 /*   force the proc number if OMP_NUM_THREADS fails  */

#include "freetwm_flags.c"
#include "freetwm_types.c"
#include "freetwm_utils.c"
#include "freetwm_algorithm.c" 
#include "freetwm_stochastic.c"

void mexFunction( int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[] )
{ 

 /*omp_set_num_threads(2);*/          /*   force the proc number if OMP_NUM_THREADS fails  */

  static bool firstrun = true ;       /*   Flag used to initialize  the random generator   */
  static unsigned int seed    ;       /*   Seed of the random number generator             */
 
  int i , j , k , v ;
  int m , n , p     ;
  int M , N , P     ;
  int J , Ne , ne   ;
 
  bool ret_val , Flag_Inj , Flag_Mod ;	
  
  dcmat BC ; dcvec E_in , E_out ;  /*  Boundary condition matrix, Input/output fields */
 
  double  * ptr1_x , * ptr1_y ;    /*    Pointers over   */
  double  * ptr2_x , * ptr2_y ;    /*    matlab input    */
  double  * ptr3_x , * ptr3_y ;    /*  output arguments  */
  double  * ptr_mod           ;

  const mwSize * arg_dim ; 

  mwSize dim[3] ;

  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* ----------------some constant, possibly user defined seed for the stochastic part ------------------------------------- -------------- */
  /* -----------------  is used to initialize the mkl or the gcc random generators -------------------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */
    
#if HAVEMKL == 1
  
  static VSLStreamStatePtr rng_stream ; 

  if( firstrun ) 
    {
      seed = SEED ; 
      if(DBG>0) mexPrintf("\n--First run --- Random number generator initialized with seed %i --\n",seed); 
      vslNewStream(&rng_stream,UNI_RNG_METHOD,seed) ;
      firstrun = false ;
    }
  
#else
 
  if( firstrun ) 
    {
      seed = SEED ;
      if(DBG>0) mexPrintf("\n--First run --- Random number generator initialized with seed %i --\n",seed); 
      srand(seed); 
      firstrun = false ;
    }
  
#endif
    
  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* -----------------------------------------Start Processing the command line ----------------------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */

  const int  K = mxGetScalar( prhs[0] ) ;  /* steps    */
  const int  V = mxGetScalar( prhs[1] ) ;  /* sections */
  const int  S = mxGetScalar( prhs[2] ) ;  /* sampling */
 
  if(DBG>0) mexPrintf("\nPerforming %i Steps of integration, expecting %i section(s) and sampling output every %i \n",K,V,S);

  if( V < 1 || K < 1 )               { mexPrintf("\t\t\t\t --error-- Step nb & waveguide number cannot be < 1         \n"  ) ; mexErrMsgTxt(""); }
  if( nrhs < 3*V+4 || nrhs > 3*V+6 ) { mexPrintf("\t\t\t\t --error-- Nb of input  argument incorrect for %i Waveguides\n",V) ; mexErrMsgTxt(""); }
  if( nlhs != V + 1  )               { mexPrintf("\t\t\t\t --error-- Nb of output argument incorrect for %i Waveguides\n",V) ; mexErrMsgTxt(""); }

  section ** sim = ( section ** ) malloc( (size_t)(V)*sizeof(section*) ) ;   /* Allocate pointers over V sections */

  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* -----------------------go over the  mex right hand side, doing some consistency checks   --------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */

  for( i = 0 ; i < V ; i++ )                                                
    {
      Is_A_Mesh( & M , & N , prhs , 3+3*i ) ;                     /*-- Is it a 3 dimensional complex arrays MxNx6 that represents a mesh ? -*/

      ret_val = Is_A_Matlab_Matrix( "real" , "abort_on_failure" , PAR_NUM , N , prhs , 4+3*i ) ;
                                                                  /*-- Is it a 2 dimensional real array PAR_NUMxN that represents param ? --*/

      J  =  mxGetScalar( prhs[5+3*i] )  ;  
      if( M > J && N > J && (N-1)%J == 0 && J > 1 )               /*------------- Can the decimation factor can be applied ? ---------------*/
	{  if(DBG>0) mexPrintf("section %i is composed of %i points and is eligible for speedup with a decimation factor %i \n",i+1,N,J); }
      else J = 1 ;
  
      sim[i] = allocate_section(  M ,  N , J ) ;                  /*--- Since everything is correct, let's allocate and fill the arrays ----*/
   
      Initialize_Section_IC (  sim[i] ,  prhs[3+3*i] ) ;          /*------------------ fill the initial condition mesh  --------------------*/

      Initialize_Section_PAR(  sim[i] ,  prhs[4+3*i] ) ;          /*------------------ fill the parameter matrix ---------------------------*/

      if ( sim[i]->Passive == true )
	{
	  if ( DBG>0 )  mexPrintf("Section %i is passive, only internal losses and distributed coupling are considered \n", i+1 ) ;
	}
      else                                                        /*----------- Initialize the lookup table for the convolution kernel -----*/
	{
	  if ( DBG>0 ) mexPrintf("Initializing Lookup table for Section : %i\n",i+1) ; Initialize_Section_LUT(  sim[i]  ) ; 
	}
    }
 
  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* ----------------------------------  Next, let's process the BC matrix, of size 2Vx2V     --------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */
 
  ret_val = Is_A_Matlab_Matrix( "complex" , "abort_on_failure" , 2*V , 2*V , prhs , 3+3*V ) ;  /*---- is BC a (2V)x(2V) complex matrix ? ---*/

  BC = allocate_dcmat( 2*V , 2*V ) ; 
  
  ptr1_x = mxGetPr( prhs[3+3*V] ) ,  ptr1_y = mxGetPi( prhs[3+3*V] ) ; 
  
  for(n=0;n<2*V;n++) for(m=0;m<2*V;m++) { j = 2*V*n + m ; BC[m][n] = ptr1_x[j] + I * ptr1_y[j] ; }   
  
  /* --------------------------------------------------------------------------------------------------------------------------------------- */
  /* ----------------------------- Process the injection fields and the parameter modulation------------------------------------------------ */
  /* ---------  check if we do have injection and/or modulation, the dimensions of the matrix & point ptr2_xy to the injection field-------- */
  /* --------------------------------------------------------------------------------------------------------------------------------------- */

  if(nrhs == 3*V+4 )
    {  
      Flag_Inj = false ;  Flag_Mod = false ;  
      if(DBG>0)	{ mexPrintf("Inj. matrix : No\n") ;  mexPrintf("Modulation matrix : No\n") ;   }
    }

  if(nrhs == 3*V+5 )
    {
      Flag_Inj = Is_A_Matlab_Matrix( "complex" , "no_abort_on_failure" , K , 2 , prhs , 3*V + 4 ) ;  /*-- is Inj a Kx2 complex matrix ? -----*/
      Flag_Mod = Is_A_Matlab_Matrix( "real"    , "no_abort_on_failure" , K , 3 , prhs , 3*V + 4 ) ;  /*-- is Mod a Kx3 real    matrix ? -----*/
     
      if( Flag_Inj == true ) 
	{
	  ptr2_x = mxGetPr( prhs[3*V+4] ) ,  ptr2_y = mxGetPi( prhs[3*V+4] ) ;  
	  if(DBG>0) { mexPrintf("Inj. matrix : Yes\n") ; mexPrintf("Mod. matrix : No \n") ; }
	}

      if( Flag_Mod == true ) 
	{
	  ptr_mod =  mxGetPr( prhs[3*V+4] ) ; 
       	  if(DBG>0) { mexPrintf("Inj. matrix : No \n") ; mexPrintf("Mod. matrix : Yes\n") ; }
	}
     
      if ( Flag_Inj == false && Flag_Mod == false  ) 
	{  
	  Flag_Inj = Is_A_Matlab_Matrix( "complex" , "abort_on_failure" , K , 2 , prhs , 3*V + 4 ) ;  /*-- is Inj a Kx2 complex matrix ? ----*/
	  Flag_Mod = Is_A_Matlab_Matrix( "real"    , "abort_on_failure" , K , 3 , prhs , 3*V + 4 ) ;  /*-- is Mod a Kx3 real    matrix ? ----*/
	}
    }

  if(nrhs == 3*V+6 )
    {
      Flag_Inj = Is_A_Matlab_Matrix( "complex" , "abort_on_failure" , K , 2 , prhs , 3*V + 4 ) ;  /*---- is Inj a Kx2 complex matrix ? ------*/
      Flag_Mod = Is_A_Matlab_Matrix( "real"    , "abort_on_failure" , K , 3 , prhs , 3*V + 5 ) ;  /*---- is Mod a Kx3 real    matrix ? ------*/

      ptr2_x = mxGetPr( prhs[3*V+4] ) ,  ptr2_y = mxGetPi( prhs[3*V+4] ) , ptr_mod =  mxGetPr( prhs[3*V+5] )  ; 

      if(DBG>0) { mexPrintf("Inj. matrix : Yes\n") ; mexPrintf("Modulation matrix : Yes\n") ; }
    } 

  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* ---------  create a Kx6xV complex array in order to save Em(0),Ep(L),D0(0),D0(L),D2(0),D2(L) for each waveguide  --------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */

  int PTS=(int)ceil((double)K/S); if(DBG>0) mexPrintf("allocating %i pts x %i x %i for saving output\n\n",PTS,6,V) ; 

  dim[0] = PTS ; 
  dim[1] = 6 ; 
  dim[2] = V ;  
 
  plhs[0] = mxCreateNumericArray(3, dim, mxDOUBLE_CLASS, mxCOMPLEX ) ;  
 
  ptr1_x  =(double *) mxGetData( plhs[0] ) , ptr1_y  =(double *) mxGetImagData( plhs[0] ) ; 
 
  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* -------------------------  allocate two V vectors for linking E_in and E_out via BC--------------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */

  E_in  = allocate_dcvec( 2*V ) ;   
  E_out = allocate_dcvec( 2*V ) ;

  /* -------------------------------------------------------------------------------------------------------------------------------------- */
  /* ---------------------------------------------Time integration start here ------------------------------------------------------------- */
  /* -------------------------------------------------------------------------------------------------------------------------------------- */

  if(DBG>0) mexPrintf("simulation starting .... \n");

  /* loop over the step number */

  i = 0 ;
  for( k = 0 ; k < K ; k++ ) 
    {      
      /* update output fields at the facet of all waveguides */

      for( v = 0 ; v < V ; v++ )	                                      
	{
	  Ne = sim[v]->Ne ;

	  REAL complex hdbr_m = sim[v]->Par[1][1 ] - I * sim[v]->Par[2][1 ] ;
	  REAL complex hdbr_p = sim[v]->Par[1][Ne] + I * sim[v]->Par[2][Ne] ;

	  E_out[ 2*v   ] = ( 1.-0.5*sim[v]->Par[0][1 ] ) * sim[v]-> Em[0][1 ] + (0.5*I) * ( sim[v]->Pm[0][1 ] - hdbr_m * sim[v]->Ep[0][1 ] ) ;
	  E_out[ 2*v+1 ] = ( 1.-0.5*sim[v]->Par[0][Ne] ) * sim[v]-> Ep[0][Ne] + (0.5*I) * ( sim[v]->Pp[0][Ne] - hdbr_p * sim[v]->Em[0][Ne] ) ;     
 	}

      /*  If the trigger of the sampling points eval to true, save the output Em(0),Ep(1) pts for all waveguides */

      if( k%S == 0 ) 
	{
	  for( v = 0 ; v < V ; v++ )	                                      
	    { 
	      Ne =sim[v]->Ne ;
		 
	      j = PTS*(0) + i + v * ( 6 * PTS ) ; ptr1_x[j] = creal( E_out[ 2*v   ] )   ; ptr1_y[j] = cimag(  E_out[ 2*v   ] )   ;
	      j = PTS*(1) + i + v * ( 6 * PTS ) ; ptr1_x[j] = creal( E_out[ 2*v+1 ] )   ; ptr1_y[j] = cimag(  E_out[ 2*v+1 ] )   ;

	      j = PTS*(2) + i + v * ( 6 * PTS ) ; ptr1_x[j] =       sim[v]-> D0[0][1 ]  ; ptr1_y[j] = 0.                         ;
	      j = PTS*(3) + i + v * ( 6 * PTS ) ; ptr1_x[j] =       sim[v]-> D0[0][Ne]  ; ptr1_y[j] = 0.                         ;
	      j = PTS*(4) + i + v * ( 6 * PTS ) ; ptr1_x[j] = creal(sim[v]-> D2[0][1 ]) ; ptr1_y[j] = cimag(sim[v]-> D2[0][1 ])  ;
	      j = PTS*(5) + i + v * ( 6 * PTS ) ; ptr1_x[j] = creal(sim[v]-> D2[0][Ne]) ; ptr1_y[j] = cimag(sim[v]-> D2[0][Ne])  ;
	    }
	  i++;
	}

      for( v = 0 ; v < V ; v++ )                                       
	{    
	  Ne = sim[v]->Ne ;  J = sim[v]->J ;                                
	  
	  /* We find the inputs at the facet from the BC  */
	  
	  E_in[2*v  ] =  E_in[2*v+1] = 0. ;
	  
	  for( j = 0 ; j < 2*V ; j++ ) 
	    {
	      E_in[2*v]   += BC[2*v  ][j] * E_out[j] ;
	      E_in[2*v+1] += BC[2*v+1][j] * E_out[j] ;
	    }
	  
	  /*  and assign the fake pts Ep(0) and Em(N+1) in the past value J-1 (i.e. 0 if normal and J-1 if decimated) */
	  
	  sim[v]-> Ep[J-1][ 0  ] = E_in[2*v  ] ;
	  sim[v]-> Em[J-1][Ne+1] = E_in[2*v+1] ;
	}
      
      /* to the first waveguide E+(0) and to the last waveguide E-(L) we put the injection fields, if we have some */
      
      if( Flag_Inj  == true )
	{
	  v =   0 ; Ne = sim[v]->Ne ; J = sim[v]->J ; sim[v]-> Ep[J-1][  0 ] += ptr2_x[ k     ] + I * ptr2_y[ k     ] ;
	  v = V-1 ; Ne = sim[v]->Ne ; J = sim[v]->J ; sim[v]-> Em[J-1][Ne+1] += ptr2_x[ k + K ] + I * ptr2_y[ k + K ] ;
	}

      /* update in time all the waveguides */

      for( v = 0 ; v < V ; v++ ) 
	{ 
	  if( sim[v]->Passive == false ) 
	    {
	      if( v == 0 && Flag_Mod == true ) 	Section_Update ( sim[v] , ptr_mod[ k ] , ptr_mod[ k + K ] , ptr_mod[ k + 2*K ] ) ; 
	      else                              Section_Update ( sim[v] ,       0.     ,         0.       ,        0.          ) ;
	    }
	  else Section_Update_Passive ( sim[v] ) ;

	  section_shift ( sim[v] ) ; 

	  /* Stochastic part, put noise in the polarization */

#if HAVEMKL==1  
	  if( sim[v]->Passive == false ) Section_Shake_mkl( sim[v] , rng_stream ) ; 
#else
	  if( sim[v]->Passive == false ) Section_Shake_gcc( sim[v] ) ; 
#endif
	}  
    }

  /* ------------------------------------------------------------------------------------------------------------------------------------- */
  /* -------------------------------------------End of time integration ------------------------------------------------------------------ */
  /* ------------------------------------------------------------------------------------------------------------------------------------- */
 
  if(DBG>0) mexPrintf("simulation done ........\n\n---------------:setting up section(s) output(s) \n");
  
  /* ------------------------------------------------------------------------------------------------------------------------------------- */
  /* ------------------------------------------------- Set output Section----------------------------------------------------------------- */
  /* ------------------------------------------------------------------------------------------------------------------------------------- */

  for( v = 0 ; v < V ; v++ )
    { 

      dim[0] = M = sim[v]->M ; dim[1] = N = sim[v]->N ;  dim[2] = 6 ; Ne = sim[v]->Ne ; J = sim[v]->J ;
     
      if(DBG>0) mexPrintf("\t\tsetting up section output of waveguide nb %i of dims %ix%ix%i\n",v+1,M,N,6);
     
      plhs[1+v] = mxCreateNumericArray(3, dim, mxDOUBLE_CLASS, mxCOMPLEX ) ; /* all guys initialized to zero */

      ptr3_x  = (double *)mxGetData(plhs[1+v]) , ptr3_y =  (double *)mxGetImagData(plhs[1+v]) ; 
   
      for( ne = 1 ; ne <= Ne ; ne++ )
      	for( m = 0 ; m < M ; m++ )
      	  {
      	    j= M*(ne-1)*J + m + 0*M*N ;  ptr3_x[j] = creal( sim[v]-> Ep[m][ne] ) ; ptr3_y[j] = cimag( sim[v]-> Ep[m][ne] ) ;
	    j= M*(ne-1)*J + m + 1*M*N ;  ptr3_x[j] = creal( sim[v]-> Em[m][ne] ) ; ptr3_y[j] = cimag( sim[v]-> Em[m][ne] ) ;
      	    j= M*(ne-1)*J + m + 2*M*N ;  ptr3_x[j] = creal( sim[v]-> Pp[m][ne] ) ; ptr3_y[j] = cimag( sim[v]-> Pp[m][ne] ) ;
      	    j= M*(ne-1)*J + m + 3*M*N ;  ptr3_x[j] = creal( sim[v]-> Pm[m][ne] ) ; ptr3_y[j] = cimag( sim[v]-> Pm[m][ne] ) ;
      	    j= M*(ne-1)*J + m + 4*M*N ;  ptr3_x[j] = creal( sim[v]-> D0[m][ne] ) ; ptr3_y[j] = 0. ;  /*---- Im(D0)=0. ----*/ 
      	    j= M*(ne-1)*J + m + 5*M*N ;  ptr3_x[j] = creal( sim[v]-> D2[m][ne] ) ; ptr3_y[j] = cimag( sim[v]-> D2[m][ne] ) ;
      	  }

    }

  /* ----------------------------------------------------------------------------------------------------------------------------------- */
  /* --------------------------------------------------free the mem -------------------------------------------------------------------- */
  /* ----------------------------------------------------------------------------------------------------------------------------------- */

  if(DBG>0) mexPrintf("\nFreeing memory .........\n\n");  

  for( v = 0 ; v < V ; v++ ) free_section( sim[v] ) ; free(sim);
 
  free_dcmat( BC , 2*V );

  free( E_in  ) ;
  free( E_out ) ;

  return ;
}

/* ------------------------------------------------------------------------------------------------------------------------------------- */
/* ---------------------------------------------------- EOF----------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------------------------------------------------------------------- */
