/*
----------------------------Authors -----------------------------------

Julien Javaloyes
Salvador Balle

Universitat de les Illes Baleares
Palma de Mallorca

-----------------------------Copyright --------------------------------

This file is part of Freetwm

Freetwm is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 3 of the License, or (at your
option) any later version.

Freetwm is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Freetwm; see the file COPYING.  
If not, see <http://www.gnu.org/licenses/>.

-----------------------------------------------------------------------
*/

#define PAR_NUM 16

/* ---------------------------------------------------------------------------------------------------- */
/* ---------------------------Some type and some struct  ---------------------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

typedef REAL complex ** dcmat ;
typedef REAL         **  dmat ;
typedef REAL complex  * dcvec ;
typedef REAL          *  dvec ;

typedef struct 
{  
  int M  ;       /* -----------point in the past, 0 is current, M-1 is last------------------------ */
  int N  ;       /* -----------point in space from 1 to N. 0 and N+1 used for BC------------------- */
  int J  ;       /* -----------decimation factor, jump every J spatial points---------------------- */ 
  int Ne ;       /* --effective points in space Ne=(N-1)/J+1, from 1 to Ne, 0 and Ne+1 use for BC-- */
  bool Passive ; /* -----------Flag for deciding if the section is passive or not ----------------- */

  dcmat Ep ;
  dcmat Em ;

  dcmat Pp ;
  dcmat Pm ;

  dmat  D0 ;
  dcmat D2 ; 

  dcmat  chi ;
  dcmat dchi ;

  dmat  Par ;

#if APPROX==0  

  dcmat Dp2Em ;
  dcmat Dm2Ep ; 

#endif

} section ;

/* --------------------------------------------------------------------------------------------- */
/* ------------------allocate a double vector -------------------------------------------------- */
/* --------------------------------------------------------------------------------------------- */

dvec  allocate_dvec ( int M )
{ 
  if(DBG>1) mexPrintf("allocate a  dvec (%i)\n",M); 
  return (REAL *) calloc( M , sizeof( REAL ) ) ; 
}

/* ----------------------------------------------------------------------------------------------------- */
/* ------------------allocate a complex double vector -------------------------------------------------- */
/* ----------------------------------------------------------------------------------------------------- */

dcvec allocate_dcvec( int M )
{ 
  if(DBG>1) mexPrintf("allocate a dcvec (%i)\n",M); 
  return (REAL complex *) calloc( M , sizeof( REAL complex) ) ; 
}

/* ----------------------------------------------------------------------------------------------------- */
/* ------------------allocate a complex double  matrix with subscript range m[0..M-1][0..N-1] ---------- */
/* ----------------------------------------------------------------------------------------------------- */

dcmat allocate_dcmat( int M , int N )
{ 
  if(DBG>1) mexPrintf("allocate a dcmat (%i,%i)\n",M,N);
 
  int m; 
 
  /* allocate pointers to rows */
 
  dcmat mat=(dcmat) malloc( (size_t)(M)*sizeof(dcvec) ) ; 

  if (!mat)  mexErrMsgTxt("allocation failure 1 in allocate_dcmat()");
  
  /* allocate rows and set pointers to them */
 
  for(m=0;m<M;m++)  
    {  
      mat[m]= allocate_dcvec( N ) ;
      if (!mat[m]) mexErrMsgTxt("allocation failure 2 in allocate_dcmat()");
    }
  return mat;

}

/* ---------------------------------------------------------------------------------------------------- */
/* ------------------allocate a double  matrix with subscript range m[0..M-1][0..N-1] ----------------  */
/* ---------------------------------------------------------------------------------------------------- */

dmat allocate_dmat( int M , int N )
{ 

  if(DBG>1) mexPrintf("allocate a dmat (%i,%i)\n",M,N);
 
  int m; 
 
  /* allocate pointers to rows */
 
  dmat mat=(dmat) malloc( (size_t)(M)*sizeof(dvec) ) ; 

  if (!mat)  mexErrMsgTxt("allocation failure 1 in allocate_dcmat()");
  
  /* allocate rows and set pointers to them */
  for(m=0;m<M;m++)  
    {  
      mat[m]= allocate_dvec( N ) ;
      if (!mat[m]) mexErrMsgTxt("allocation failure 2 in allocate_dcmat()");
    }

  return mat;

}

/* ---------------------------------------------------------------------------------------------------- */
/* ----------------------------free a complex 2D mat allocated by allocate_dcmat()--------------------  */
/* ---------------------------------------------------------------------------------------------------- */

void free_dcmat( dcmat mat , int M )
{ 
  int m ; 

  for(m=0;m<M;m++)  free( mat[m] ) ; 

  free( mat ) ; 
}

/* ---------------------------------------------------------------------------------------------------- */
/* ----------------------------free a REAL 2D mat allocated by allocate_dmat()------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

void free_dmat( dmat mat , int M )
{ 
  int m ; 

  for(m=0;m<M;m++) free( mat[m] ) ; 

  free( mat ) ; 
}

/* ---------------------------------------------------------------------------------------------------- */
/* ----------------------------------allocate a section------------------------------------------------ */
/* ---------------------------------------------------------------------------------------------------- */

section * allocate_section( const int M , const int N , const int J )
{

  int Ne = (N-1)/J + 1 ; 

  section * mysection=(section *) malloc( (size_t)sizeof(section) )  ;

  mysection->Ep  = allocate_dcmat(     M    ,  Ne + 2 ) ;  
  mysection->Em  = allocate_dcmat(     M    ,  Ne + 2 ) ;  
  mysection->Pp  = allocate_dcmat(     M    ,  Ne + 2 ) ;  
  mysection->Pm  = allocate_dcmat(     M    ,  Ne + 2 ) ;  
 
  mysection->D0  = allocate_dmat (     M    ,  Ne + 2 ) ;  
  mysection->D2  = allocate_dcmat(     M    ,  Ne + 2 ) ;  
 
  mysection->Par = allocate_dmat  ( PAR_NUM ,  Ne + 2 ) ;

  mysection-> chi = allocate_dcmat (DIM_LUT , M + 1  ) ;
  mysection->dchi = allocate_dcmat (DIM_LUT , M + 1  ) ;

#if APPROX==0 

  mysection->Dp2Em = allocate_dcmat(   M    ,  Ne + 2 ) ; 
  mysection->Dm2Ep = allocate_dcmat(   M    ,  Ne + 2 ) ;

#endif

  mysection->M  = M  ;  
  mysection->N  = N  ; 
  mysection->Ne = Ne ;
  mysection->J  = J  ;

  if(DBG>1) mexPrintf("Allocated an M=%i\tN=%i\tP=%i Section\n" , M , N , PAR_NUM ) ;
 
  return mysection;
}


/* ---------------------------------------------------------------------------------------------------- */
/* ----------------------------------free a section---------------------------------------------------- */
/* ---------------------------------------------------------------------------------------------------- */

void free_section( section * mysection )
{

  int N = mysection->N ;
  int M = mysection->M ;

  free_dcmat( mysection->Ep   ,   M     ) ; 
  free_dcmat( mysection->Em   ,   M     ) ; 
  free_dcmat( mysection->Pp   ,   M     ) ; 
  free_dcmat( mysection->Pm   ,   M     ) ;  
  
  free_dmat ( mysection->D0   ,   M     ) ; 
  free_dcmat( mysection->D2   ,   M     ) ;

#if APPROX==0 

  free_dcmat( mysection->Dp2Em,   M     ) ; 
  free_dcmat( mysection->Dm2Ep,   M     ) ;

#endif

  free_dmat ( mysection->Par  , PAR_NUM ) ; 

  free_dcmat( mysection-> chi , DIM_LUT ) ; 
  free_dcmat( mysection->dchi , DIM_LUT ) ;   
 
  free( mysection ) ;

  if(DBG>1) mexPrintf("Freed an M=%i\tN=%i\tP=%i Section\n",M,N,PAR_NUM);

}


/* ------------------------------------------------------------------------------------------------------------------------------------- */
/* ---------------------------------------------------- EOF----------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------------------------------------------------------------------- */
