function [ g_m, om_m , dg_dD , alpha ] = gnr_chi_plot( D0 , M , dt , par ) 
%  
%[ g_m , om_m , dg_dD , alpha ] = gnr_chi_plot( D0 , M , dt , [ gma , om_g , om_t , gma_c , gma_v ] ) 
%
% perform a comparison plot of the gain and the index of refraction between
% the analytical prediction and the numerical estimation the convolution integral. 
%
% The signal is assumed to be a monochromatic wave and its frequency, 
% i.e. the plot range, is from - Nyquist frequency to + Nyquist frequency.
%
% -------------------Input arguments--------------------------------------%
%
% D0    : (constant) population inversion          (a.u.)
% M     : Number of point kept in the past         (a.u.)
%
% dt    : time increment between sampling points   (s)
% gma   : linewidth of the gain                    (s^-1)
% om_g  : position of the bandgap frequency        (s^-1)
% om_t  : maximal frequency of the top of the band (s^-1)
% gma_c : Thermal linewidth of conduction electrons(s^-1)
% gma_v : Thermal linewidth of valences holes      (s^-1)
% 
%
% ------------------Output arguments--------------------------------------%
%
% g_m  : Peak gain (wrt to to a maximal saturated gain of unity)  (a.u.)
% om_m : Frequency corresponding to the peak gain                 (s^-1)
% dg_dD: Differential gain at the frequency om_m                  (a.u.)
% di_dD: Differential index variation at the frequency om_m       (a.u.)
% alpha: Alpha factor at frequency om_m                           (a.u.) 
%
% ------------------------------------------------------------------------%

if ( D0 == 0 ) , D0 = 1.e-6 ; end % in order to prevent infinite Fermi levels 
    
gma = par(1) ; om_g = par(2) ; om_t = par(3) ; gma_c =  par(4) ; gma_v = par(5) ; 

om_N = pi/dt;                                  % Dimensional Nyquist frequency
N_om = 10^3;                                   % Number of samples in omega for plotting
om   = linspace( - om_N , om_N , N_om ) ;      % plot range within the non aliased zone.
s    = dt/1000 + (0:M-1)*dt;                   % time span over which we keep the values of E(t)

om_tg = mod(om_g+om_t+om_N , 2*om_N ) - om_N ; % top of the band frequency modulo om_N

delta = 0.01;                                 % Decrement of the Fermi function and frequency variation for the adaptive quadrature 

if( gma_c < gma/100 ), gma_c = gma/100 ; disp('minimal temperature reached for electrons, using gma_c = gma/30') ; end
if( gma_v < gma/100 ), gma_v = gma/100 ; disp('minimal temperature reached for holes    , using gma_v = gma/30') ; end

%-----------------------Generate the weigths-------------------------------%

[ Wc , Fc , BFc ] = gnr_Fermi_weight( D0 , gma ,  om_t , gma_c , delta ) ; Wc = Wc + om_g ;
[ Wv , Fv , BFv ] = gnr_Fermi_weight( D0 , gma ,  om_t , gma_v , delta ) ; Wv = Wv + om_g ;

%------------------calculate the temporal kernel--------------------------%

[ chi_t , dchi_t_dD ] = gnr_chi_time_kernel( D0 , M , dt , gma ,  om_g , om_t , gma_c , gma_v , delta ) ;
 
%------------- find the numerical susceptibility chi with aliasing--------%
  
  for ii=1:numel(om) , 
      S  = chi_t.*exp( -1i*om(ii)*( 0 - s ) )     ;   % source: kernel(s) x plane wave(t-s) @ om -> E = exp(-i*omega*t)
      chi_om_num(ii) = dt * trapz ( S )           ;   % trapezoidal integration & scaling by dt
      
      S  = dchi_t_dD.*exp( -1i*om(ii)*( 0 - s ) ) ;   % source: kernel(s) x plane wave(t-s) @ om -> E = exp(-i*omega*t)
      dchi_om_dD_num(ii) = dt * trapz ( S )       ;   % trapezoidal integration & scaling by dt
  end
 
%-----------------find the exact susceptibility --------------------------%

[ chi_om , dchi_om_dD ] = gnr_chi_omega( D0 , gma , om_g , om_t , gma_c , gma_v , delta , om );

%---------------------is there a gain peak ?-------------------------------%

   g = -imag(chi_om_num) ;  % NUMERICAL  gain
 % g = -imag(chi_om    ) ;  % ANALYTICAL gain 
    
   [ valmax , I ] = max(g) ;     % position of the max
   Ig  = find(om >= om_g , 1 );  % idx of the bandgap frequency 
    
 if(  I > Ig && 2 < I < N_om-1 && g(I-1) < valmax >= g(I+1) ),   
     disp(' found a gain peak on the blue side of the bandgap! ') ; 
    g_m  = valmax ;               % gain peak value 
    om_m = om(I)  ;               % frequency of the gain peak   
    ( chi_om_num(I+1) + chi_om_num(I-1) - 2*chi_om_num(I) )/( om(I+1) - om(I) )^2
    chi_om_num(I)
    om(I)
 else
    disp('no gain peak found on the blue side of the bandgap, assuming bandgap frequency ') ;
    g_m  =  g(Ig) ;               % gain/abs value
    om_m = om(Ig) ;               % frequency of the bandgap peak
 end
          
dg_dD = - imag ( dchi_om_dD_num(I) ) ; % differential NUMERICAL gain    @ om_m
di_dD =   real ( dchi_om_dD_num(I) ) ; % differential NUMERICAL index   @ om_m

% dg_dD = - imag ( dchi_om_dD(I) )    ; % differential ANALYTICAL gain   @ om_m
% di_dD =   real ( dchi_om_dD(I) )    ; % differential ANALYTICAL index  @ om_m

alpha = -   di_dD  /  dg_dD ;          % Henry's alpha factor @ om_m   


%--------Put all these informations into a nice figure --------------------%

fig = gcf ;  set( fig , 'Position' , [0 0 1200 750] ) ;

% are we running matlab or octave 

prog = ver('OCTAVE');

if( numel(prog) == 1 ),
    
    OCTAVE = 1 ;
    
    vec_fermi = 223 ; vec_diff  = 224 ;
    vec_gain  = 221 ; vec_index = 222 ;
    
else
    
    OCTAVE = 0 ;
    
    vec_fermi = [0.0825 0.085  0.4 0.3  ] ; vec_diff  = [0.565  0.085  0.4 0.3  ] ;
    vec_gain  = [0.0825 0.4275 0.4 0.55 ] ; vec_index = [0.565  0.4275 0.4 0.55 ] ;
    
end

om_N  = om_N /1e12 ; om    = om   /1e12 ;    % scaling frequency in teraHertz
Wc    = Wc   /1e12 ; Wv    = Wv   /1e12 ;    % scaling frequency in teraHertz
om_g  = om_g /1e12 ; om_tg = om_tg/1e12 ;    % scaling frequency in teraHertz

%----------------------------Fermi level ---------------------------------%

if(OCTAVE==1), subplot(vec_fermi); else subplot('Position',vec_fermi); end 

set(gca,'FontSize',22,'LineWidth',2);

 hold on
 plot( om , Fermi( (om-om_g)*1e12 , gma_c , BFc ) ,'--b' , om , Fermi( (om-om_g)*1e12 , gma_v , BFv ) , '--r' , 'LineWidth', 2) ; 
 plot(  Wc , Fc ,'x' , Wv , Fv ,'or') ;
 plot([om_g  om_g ],[-1 1]   ,'g-.', 'LineWidth', 2 ) ;
 axis([-om_N om_N 0 1]); grid on; box on ;
 
 plot([om_g  om_g ],ylim        ,'g-.', 'LineWidth', 2 ) ;
 plot([om_tg om_tg],ylim        ,'b-.', 'LineWidth', 2 ) ;
 
 legend('e','h','Location','SouthWest');
 xlabel(' Frequency (rad.THz) '); 
 ylabel(' F( \Omega ) '        );
 
%---------------------------Gain plot ------------------------------------%

 if(OCTAVE==1), subplot(vec_gain); else subplot('Position',vec_gain); end
 
 set(gca,'FontSize',22,'LineWidth',2); hold on
 
 plot( om, -imag( chi_om    ),  'k', 'LineWidth', 2 ) ;
 plot( om, -imag( chi_om_num),'r--', 'LineWidth', 2 ) ;
 plot([om_g  om_g ],[-1 1]   ,'g-.', 'LineWidth', 2 ) ;
 plot([om_tg om_tg],[-1 1]   ,'b-.', 'LineWidth', 2 ) ;
 
 axis([-om_N om_N -1 1]);grid on; box on ;

 text( om_g  , -0.75 , '\Omega_G'          , 'FontSize' , 22 , 'Color' , 'Green' , 'HorizontalAlignment','right' ) ;
 text( om_tg ,  0.25   , '\Omega_G+\Omega_T' , 'FontSize' , 22 , 'Color' , 'Blue'  , 'HorizontalAlignment','right' ) ;
 
 ylabel(' -\Im \chi( \Omega )  ');set(gca,'XTickLabel',{''}) ;
 legend('Analytical','Numerical','Location','NorthWest');

%----------------------index  plot ---------------------------------------%

if(OCTAVE==1), subplot(vec_index); else  subplot('Position',vec_index); end 

 set(gca,'FontSize',22,'LineWidth',2); hold on
 
 plot( om , real( chi_om     ),   'k', 'LineWidth' , 2 ) ;
 plot( om , real( chi_om_num ), 'r--', 'LineWidth' , 2 ) ;  
 axis tight;
 plot([om_g  om_g ],ylim      , 'g-.', 'LineWidth' , 2 ) ;  
 plot([om_tg om_tg],ylim      , 'b-.', 'LineWidth' , 2 ) ;

 text( om_g  , -0.75 , '\Omega_G'          , 'FontSize' , 22 , 'Color' , 'Green' , 'HorizontalAlignment','right' ) ;
 text( om_tg ,  0.25   , '\Omega_G+\Omega_T' , 'FontSize' , 22 , 'Color' , 'Blue'  , 'HorizontalAlignment','right' ) ;
 
 grid on; box on ; xlim([-om_N om_N]);set(gca,'XTickLabel',{''}) ;
 ylabel(' \Re \chi( \Omega ) ');  legend('Analytical','Numerical','Location','NorthWest');
 
%---------------------differential gain-----------------------------------%

 if(OCTAVE==1), subplot(vec_diff); else subplot('Position',vec_diff); end 
 
 set(gca,'FontSize',22,'LineWidth',2); hold on ; box on ;
 
 plot( om, real(dchi_om_dD_num) , 'k-'  , 'LineWidth', 2 ) ;
 plot( om, imag(dchi_om_dD_num) , 'k--'  ,'LineWidth', 2 ) ;
 
 axis tight; xlim([-om_N om_N]); grid on;
 
 plot([om_g  om_g ],ylim        ,'g-.', 'LineWidth', 2 ) ;
 plot([om_tg om_tg],ylim        ,'b-.', 'LineWidth', 2 ) ;
 
 xlabel(' Frequency (rad.THz) ');
  ylabel('d\chi/dD( \Omega ) ');
 legend('\Re','\Im','Location','SouthWest');

end  

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%

function [ F ] = Fermi( w , gma_th , BF )
% 
% [ F ] = Fermi( w , gma_th , BF )
% 
% scaled Fermi function 1/( 1 + exp( w./gma_th - BF )
% see J. Javaloyes & S.Balle, JQE 2012 for details 
%

F= 1./( 1 + exp( w./gma_th - BF ) ) ;

end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


function [ Fi ] = Fermi_inv( y , gma_th , BF )
% 
% [ F ] = Fermi_inv( w , gma_th , BF )
% 
% scaled inverse Fermi function ( BF + log( 1./y - 1 ) ) * gma_th 
% see J. Javaloyes & S.Balle, JQE 2012 for details 
%
Fi =  ( BF + log( 1./y - 1 ) ) * gma_th  ;

end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


function [ W , F , BF , dBFdD ] = gnr_Fermi_weight( D0 , gma , om_t , gma_cv , delta )
% 
% [ W , F , BF , dBFdD ] = gnr_Fermi_weigth( D0 , gma ,  om_t , gma_cv , delta )
% 
% Generates an an unevenly spaced ensemble of frequency from zero to a maximal 
% frequency that corresponds to a smooth variation of the Fermi function. 
% Returns the scaled Fermi level coresponding to the given population inversion.
% see J. Javaloyes & S.Balle, JQE 2012 for details 
%
% -------------------Input arguments---------------------------------------
%
% D0    : (constant) population inversion            (a.u.)
% gma   : linewidth of the gain                      (s^-1)
% om_t  : maximal frequency of the top of the band   (s^-1)
% gma_cv: Thermal linewidth of electron or holes     (s^-1)
% delta : Relative decrement of the Fermi function   (a.u.)
% 
% ------------------Output arguments---------------------------------------
%
% W     : Frequencies from W(1) = 0 to W(end) = om_t (s^-1)
% F     : Corresponding values of the Fermi function (a.u.)
% BF    : Scaled Fermi level                         (a.u.)
%dBFdD  : Scaled derivative of Fermi level wrt D0    (a.u.)
%
% -------------------------------------------------------------------------

BF    = log( ( exp( D0*gma/gma_cv ) - 1 ) ) ;                                   % large band approx 
dBFdD =  (gma/gma_cv) * exp( D0*gma/gma_cv ) / ( exp( D0*gma/gma_cv ) - 1 ) ;   % large band approx 

%BF = log ( (1 -exp(-D0*gma/gma_cv))/(exp(-D0*gma/gma_cv)-exp(-om_t/gma_cv)) ) ;  % exact form

  ii=1; W(1) = 0 ; F(1) = Fermi( 0 , gma_cv , BF ) ; df = F(1) * delta ; dom_max = om_t * delta ; 
  
  while( W(ii) < om_t ),
  
      z  = F(ii) - df ;  if ( z < 0 ), w1 =  om_t ; else  w1 = Fermi_inv( z , gma_cv , BF ) ;  end 
      w2 = W(ii) + dom_max ;
    
      ii = ii+1 ; W(ii) = min( w1 , w2 ) ; F(ii) =  Fermi( W(ii) , gma_cv , BF ) ;  
     
  end

end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


function [ chi_t,  dchi_t_dD ] = gnr_chi_time_kernel( D0 , M , dt , gma ,  om_g , om_t , gma_c , gma_v , delta ) 
% 
% [ chi_t , dchi_t_dD ] = gnr_chi_time_kernel( D0 , M , dt , gma ,  om_g , om_t , gma_c , gma_v ,  delta )
% 
% Generates the convolution kernel for a given value of the population
% inversion for an an (unevenly) distributed ensemble of frequencies from 
% the gap to the top of the band that corresponds to a smooth variation
% of the Fermi function. see J. Javaloyes & S.Balle, JQE 2012 for details 
%
% -------------------Input arguments---------------------------------------
%
% D0    : (constant) population inversion             (a.u.)
% M     : Number of point kept in the past            (a.u.)
%
% dt    : time increment between sampling points      (s)
% gma   : linewidth of the gain                       (s^-1)
% om_g  : position of the bandgap frequency           (s^-1)
% om_t  : maximal frequency of the top of the band    (s^-1)
% gma_c : Thermal linewidth of conduction electrons   (s^-1)
% gma_v : Thermal linewidth of valences holes         (s^-1)
% delta : Relative decrement of the Fermi function    (a.u.)
% 
% ------------------Output arguments---------------------------------------
%
% chi_t    : Kernel from t = 0 to t = M*dt            (s^-1)
% dchi_t_dD : Kernel derivative wrt to D              (s^-1)
%
% -------------------------------------------------------------------------

% Generate the weigths 

[ Wc , Fc , BFc , dBFcdD ] = gnr_Fermi_weight( D0 , gma ,  om_t , gma_c , delta ) ;
[ Wv , Fv , BFv , dBFvdD ] = gnr_Fermi_weight( D0 , gma ,  om_t , gma_v , delta ) ;

% midpoint values of Fermi functions 

Fc_mid = 0.5 * ( Fc(2:end) + Fc(1:end-1) ) ; 
Fv_mid = 0.5 * ( Fv(2:end) + Fv(1:end-1) ) ;

% calculate the temporal kernel

 chi_t    = zeros( 1 , M ) ;
dchi_t_dD = zeros( 1 , M ) ;

for kk=1:M
    
    s    = dt*( kk - 1 + 10^(-3) ) ;
    
    eiWc = exp(-1i*s*Wc )   ; 
    eiWv = exp(-1i*s*Wv )   ;
   
    chi_t(kk) =                 sum( ( eiWc(2:end) - eiWc(1:end-1) ).*Fc_mid )     ; % electrons  contribution
    chi_t(kk) =  chi_t(kk)   +  sum( ( eiWv(2:end) - eiWv(1:end-1) ).*Fv_mid )     ; % holes      contribution
    chi_t(kk) =  chi_t(kk)   +   1 - exp( -1i*om_t*s )                             ; % background contribution
    chi_t(kk) =  chi_t(kk) / ( -1i*s ) * exp( - ( gma + 1i*om_g ) * s )            ;
    
    dchi_t_dD(kk) =                 sum( ( eiWc(2:end) - eiWc(1:end-1) ).*Fc_mid.*(1 - Fc_mid) ) * dBFcdD  ; % electrons  contribution
    dchi_t_dD(kk) = dchi_t_dD(kk) + sum( ( eiWv(2:end) - eiWv(1:end-1) ).*Fv_mid.*(1 - Fv_mid) ) * dBFvdD  ; % holes      contribution
    dchi_t_dD(kk) = dchi_t_dD(kk)/( -1i*s ) * exp( - ( gma + 1i*om_g ) * s )       ;
    
end
      
 chi_t    = -1i* chi_t   /pi ;
dchi_t_dD = -1i*dchi_t_dD/pi ;

end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%


function [ chi_om , dchi_om_dD ] = gnr_chi_omega( D0 ,  gma ,  om_g , om_t , gma_c , gma_v , delta , om )
% 
% [ chi_om , dchi_om_dD ] = gnr_chi_omega( D0 ,  gma ,  om_g , om_t , gma_c , gma_v , delta , om )
% 
% Generates the susceptibility for a given value of the population
% inversion for an an (unevenly) distributed ensemble of NF frequencies
% from zero to the top of the band that corresponds to a smooth variation
% of the Fermi function. 
% see J. Javaloyes & S.Balle, JQE 2012 for details 
%
% -------------------Input arguments---------------------------------------
%
% D0    : (constant) population inversion             (a.u.)
% gma   : linewidth of the gain                       (s^-1)
% om_g  : position of the bandgap frequency           (s^-1)
% om_t  : maximal frequency of the top of the band    (s^-1)
% gma_c : Thermal linewidth of conduction electrons   (s^-1)
% gma_v : Thermal linewidth of valenlces holes        (s^-1)
% delta : Relative decrement of the Fermi function    (a.u.)
% om    : frequency                                   (s^-1)
% 
% ------------------Output arguments---------------------------------------
%
% chi_om     : susceptibility      from om(1) to om(end)       (a.u)
% dchi_om_dD : derivative wrt to D from om(1) to om(end)       (a.u)
%
% -------------------------------------------------------------------------

% Generate the weigths

[ Wc , Fc , BFc , dBFcdD ] = gnr_Fermi_weight( D0 , gma , om_t , gma_c , delta ) ;
[ Wv , Fv , BFv , dBFvdD ] = gnr_Fermi_weight( D0 , gma , om_t , gma_v , delta ) ;

% midpoint values of Fermi functions

Fc_mid = 0.5 * ( Fc(2:end) + Fc(1:end-1) ) ;
Fv_mid = 0.5 * ( Fv(2:end) + Fv(1:end-1) ) ;

% find the exact susceptibility

K=numel(om) ; chi_om = zeros(1,K) ; dchi_om_dD = zeros(1,K) ;

for ii=1:K
    
    chi_om(ii) =              sum( ( log( gma +1i*( Wc(2:end) - om(ii) + om_g ) ) - log ( gma +1i*( Wc(1:end-1) - om(ii) + om_g ) ) ).*Fc_mid ) ;
    chi_om(ii) = chi_om(ii) + sum( ( log( gma +1i*( Wv(2:end) - om(ii) + om_g ) ) - log ( gma +1i*( Wv(1:end-1) - om(ii) + om_g ) ) ).*Fv_mid ) ;
    chi_om(ii) = chi_om(ii) - log(  gma +1i*( om_t - om(ii) + om_g ) ) + log ( gma - 1i* ( om(ii) - om_g ) ) ;
   
    dchi_om_dD(ii) =                  sum( ( log( gma +1i*( Wc(2:end) - om(ii) + om_g ) ) - log ( gma +1i*( Wc(1:end-1) - om(ii) + om_g ) ) ).*Fc_mid.*(1-Fc_mid)*dBFcdD ) ;
    dchi_om_dD(ii) = dchi_om_dD(ii) + sum( ( log( gma +1i*( Wv(2:end) - om(ii) + om_g ) ) - log ( gma +1i*( Wv(1:end-1) - om(ii) + om_g ) ) ).*Fv_mid.*(1-Fv_mid)*dBFvdD ) ;
     
end

  chi_om    =  chi_om    / (-pi) ; 
 dchi_om_dD = dchi_om_dD / (-pi) ;
 
end

%-------------------------------------------------------------------------%
%-------------------------------------------------------------------------%
