function [ M , N , dt ] = gnr_good_discr( tau_c , par )
%  
% --- [ M , N , dt ] = gnr_good_discr( tau_c , par )
%    
%    Generates good values for the spatial discretization (N) and for the 
%    number of point to be kept from the past history (M) according to 
%    the value of the section propagation time tau_c (in seconds) and from 
%    the parameters par(1:16) expressed in inverse seconds. 
%    
%    In case where one wants to find the proper discretization for a 
%    structure composed of several section V: par must be a matrix 16xV 
%    and tau_c a vector of V elements. Here, M and N become V elements vectors.
%
%    The function perform some check over the parameter values 
%    given as input and may detect some errors.
%
% -------------------Input arguments--------------------------------------
%
%   tau_c -  propagation time within each section (Vx1 or 1xV)
%
%  Parameters is a matrix 16xV such that each line correspond to
%
%  Waveguide parameters      : 
%    1 -  losses   
%    2 -  Real(dfb)  
%    3 -  Imag(dfb) 
%
%  Active material parameters :
%    4 -  gain       
%    5 -  bandwidth 
%    6 -  bandgap   
%    7 -  topband   
%    8 -  gma_c        
%    9 -  gma_v         
%   10 -  Sat intensity of the medium
%
%  Carrier parameters        : 
%    11 -  Current  
%    12-  A         
%    13 - B         
%    14 - C         
%    15 - 4Dq^2     
%    16 - Spontaneous Emission rate 
%
% ------------------Output arguments--------------------------------------
%
%    M  -  Number of point to be kept in the past (1xV) 
%    N  -  Number of point that discretize the section  (1xV)        
%    dt - associated time step   
%
% ------------------------------------------------------------------------

[ alpha , I_alpha ] = max( par( 1,:) ) ;
[ dfb_x , I_dfb_x ] = max( par( 2,:) ) ;
[ dfb_y , I_dfb_y ] = max( par( 3,:) ) ;

[ chi0  , I_chi0  ] = max( par( 4,:) ) ; 
[ gma   , I_gma   ] = max( par( 5,:) ) ;
% [ om_g  , I_om_g  ] = max( par( 6,:) ) ;
[ om_t  , I_om_t  ] = max( par( 7,:) ) ;
[ gma_c , I_gma_c ] = max( par( 8,:) ) ;
[ gma_v , I_gma_v ] = max( par( 9,:) ) ;

[ J     ,   I_J  ]  = max( par(11,:) ) ;
[ A     ,   I_A  ]  = max( par(12,:) ) ;
[ B     ,   I_B  ]  = max( par(13,:) ) ;
[ C     ,   I_C  ]  = max( par(14,:) ) ;
[ D4Q2  ,  I_D4Q2]  = max( par(15,:) ) ;

dt =  0.897 * pi / om_t  ;                 % Heuristic choice of the temporal step

N = ceil( tau_c/dt ) ;                     % appropriate mesh size 
    
M = ceil ( 6 ./ ( par( 5,:) * dt )  ) ;    % Heuristic choice of the past segment of history (M)


if ( alpha * dt > 0.1 ), disp(['Problem in section ' num2str(I_alpha)]); disp('very large propagation losses       ,  check for error')  ; end
if ( dfb_x * dt > 0.1 ), disp(['Problem in section ' num2str(I_dfb_x)]); disp('very large distributed coupling (Re),  check for error')  ; end
if ( dfb_y * dt > 0.1 ), disp(['Problem in section ' num2str(I_dfb_y)]); disp('very large distributed coupling (Im),  check for error')  ; end

if ( chi0  * dt > 0.1 ), disp(['Problem in section ' num2str(I_chi0)]); disp('very large gain             , check for error')  ; end
if ( gma   * dt > 0.3 ), disp(['Problem in section ' num2str(I_gma )]); disp('very large intraband rate   , check for error')  ; end

if ( J     * dt > 0.1 ), disp(['Problem in section ' num2str(I_J   )]); disp('very large current          , check for error')  ; end
if ( A     * dt > 0.1 ), disp(['Problem in section ' num2str(I_A   )]); disp('very fast recovery time (A) , check for error')  ; end
if ( B     * dt > 0.1 ), disp(['Problem in section ' num2str(I_B   )]); disp('very fast recovery time (B) , check for error')  ; end
if ( C     * dt > 0.1 ), disp(['Problem in section ' num2str(I_C   )]); disp('very fast recovery time (C) , check for error')  ; end
if ( D4Q2  * dt > 0.1 ), disp(['Problem in section ' num2str(I_D4Q2)]); disp('very fast diffusion (4Dq0^2), check for error')  ; end

if(   om_t < 10*gma   ), disp(['Problem in section ' num2str(I_om_t)]); disp('very narrow band ( om_t < 10gma ),  check for error')  ; end

if ( gma_c >    om_t  ), disp(['Problem in section ' num2str(I_gma_c)]); disp('too large effective temperature of the electrons with respect to the size of the band, check for errors ')  ; end
if ( gma_v >    om_t  ), disp(['Problem in section ' num2str(I_gma_v)]); disp('too large effective temperature of the holes     with respect to the size of the band, check for errors ')  ; end

if ( gma_c < gma/100. ), disp(['Warning in section ' num2str(I_gma_c)]); disp('minimal temperature reached for electrons distribution, the program will use gma_c = gma/100 ')  ; end
if ( gma_v < gma/100. ), disp(['Warning in section ' num2str(I_gma_v)]); disp('minimal temperature reached for     holes distribution, the program will use gma_v = gma/100 ')  ; end



end
