function [ maxtab , mintab , momtab ] = peakdet ( y , itmax , itmin  , x )

%[ MAXTAB , MINTAB , MOMTAB ] = peakdet(y, itmax, itmin )
%
%         finds the local  maxima and minima ("peaks") in a vector y.
%         MAXTAB and MINTAB consists of two columns. Column 1
%         contains indices in y, and column 2 the found values.
%       
%         With [MAXTAB, MINTAB] = PEAKDET( y , itmax, itmin , x ) 
%         the indices in MAXTAB and MINTAB are replaced 
%         with the corresponding x-values.
% 
%         A point is considered a maximum peak if it has the maximal
%         value, and was preceded (and followed) by itmax lower values 
%
%         Similarly, a point is considered a minima if it has a minimal
%         value, and was preceded (and followed) by itmin larger values 
%
%         This  recursive decimation method ensures (or enforces) 
%         that each maxima (resp. minima) is the absolute one within 
%         a region of width  2*itmax (resp. 2*itmin) around it. 
%
%         If itmin is ommited the min are not calculated 
%
%         If given a third output argument, the array MOM consists of four
%         columns that contain the energy/integral of the pulses peaking 
%         at the maxima, the time at 50% of the energy, the fwhm, 
%         as deduced from the second cumulant and finally the skewness. 
%
maxtab = []; mintab = [];

y = y(:) ; % Just in case this wasn't a proper vector

K = numel(y) ;

if exist('x','var') == 0,
    disp('no time array given, creating equispaced index');  x = (1:K)';
else
    x = x(:) ;
end

if(length(x) ~= K) , error('y and x size not compatible'); end
  
if (length(itmax(:)))>1
  error('Input argument itmax must be a scalar');
end


if itmax < 1
  error('Input argument itmax must be at least one');
end

rng= 1+itmax : K-itmax ;

pos=zeros(K,1);

% find the maxima

disp('Searching for the maxima');

for ii=1:itmax, pos(rng) = (y(rng) > y(rng-ii) ) .* (y(rng) > y(rng+ii) ) ; rng = find(pos==1); end

% save the position and values of the maxima

maxtab(:,1) = x(rng) ;
maxtab(:,2) = y(rng) ;

if nargout == 3
    disp('third argument given, searching for the first three moments')
    for ii=1:numel(rng) ,
        k   = (rng(ii)-itmax:rng(ii)+itmax-1) ;
        
        dx  =       x(k+1) - x(k)  ;
        
        ym = y(k+1)                              + y(k)                            ; mu0  = 0.5*sum( ym.*dx )       ;
        ym = y(k+1).*  x(k+1)                    + y(k).* x(k)                     ; mu1  = 0.5*sum( ym.*dx ) / mu0 ;
        ym = y(k+1).*( x(k+1)-mu1           ).^2 + y(k).*( x(k)-mu1           ).^2 ; mu2  = 0.5*sum( ym.*dx ) / mu0 ;
        ym = y(k+1).*((x(k+1)-mu1)/sqrt(mu2)).^3 + y(k).*((x(k)-mu1)/sqrt(mu2)).^3 ; mu3  = 0.5*sum( ym.*dx ) / mu0 ;
        ym = y(k+1).*((x(k+1)-mu1)/sqrt(mu2)).^4 + y(k).*((x(k)-mu1)/sqrt(mu2)).^4 ; mu4  = 0.5*sum( ym.*dx ) / mu0 ;
        ym = y(k+1).*((x(k+1)-mu1)/sqrt(mu2)).^5 + y(k).*((x(k)-mu1)/sqrt(mu2)).^5 ; mu5  = 0.5*sum( ym.*dx ) / mu0 ;

        momtab(ii,1) = mu0 ;
        momtab(ii,2) = mu1 ;
        momtab(ii,3) = 2.35482 * sqrt(mu2) ;
        momtab(ii,4) = mu3     ;
        momtab(ii,5) = mu4     ;
        momtab(ii,6) = mu5     ;

        
    end
    
end

if exist('itmin','var')
    
if length(itmin(:))>1, error('Input argument itmin must be a scalar')    ; end
if itmax < 1         , error('Input argument itmax must be at least one'); end
    
rng= 1+itmin : K-itmin ;

for ii=1:itmin, pos(rng) = (y(rng) < y(rng-ii) ) .* (y(rng) < y(rng+ii) ) ; rng = find(pos==1); end

mintab(:,1) = x(rng) ;
mintab(:,2) = y(rng) ;

end 

end

