function [ msh ] = unfold( msh , D  , PAR ) 
%
%[ msh ] = unfold( msh , D , PAR ) 
% unfold a decimated mesh into a normal one by interpolating the missing points.
% Field interpolation is done via an Euler step along the forward/backward characteristics 
% from the past stored values of the field and the polarization.
% The polarisation and the carriers profile are simply interpolated linearly in space. 
%
% msh  :  Mesh of size MxNx6
% D    :  Decimation factor (<M)
% PAR  :  Matrix of the spatially varying parameters 16xN
%
%

[ M , N , V ] = size( msh ) ;

%% Unfold Ep from left to right with EULER step to make it nice and continuous

l=1-D;

for j=1:N,
    
      k = mod( j - 1 , D ) ;
      if(k==0), l = l + D ;  end
    
      msh(1,j,1) = ( 1 - k*PAR(1,l) ) * msh(k+1,l,1) ... internal losses
         + k*( PAR(2,l) + 1i*PAR(3,l) )*msh(k+1,l,2) ... DFB coupling 
                               + k*1i*msh(k+1,l,3) ; ... Polarisation contribution
  
end

%% Unfold Em from right to left

l=N+D;

for j=N:-1:1,
    
      k = mod( 1 - j , D ) ;
      if(k==0), l = l - D ; end
      
       msh(1,j,2) = ( 1 - k*PAR(1,l) ) * msh(k+1,l,2) ... % internal losses
          + k*( PAR(2,l) + 1i*PAR(3,l) )*msh(k+1,l,1) ... % DFB coupling 
                                + k*1i*msh(k+1,l,4) ; ... Polarisation contribution 
                                   

end

%% interpolate the polarisation Pp and Pm and the carriers D0

% The polarisation P+ (P-), proportional to E+ (E-) being fast evolving, we interpolate the slow amplitude

slow_pp= msh(:,:,3)./msh(:,:,1);

slow_pm= msh(:,:,4)./msh(:,:,2);

% The grating being proportional to E+E-^s being fast evolving, we interpolate the slow amplitude

slow_var= msh(:,:,6)./msh(:,:,1)./conj(msh(:,:,2)); %N2/(E+E-^s)


for j=1:(N-1),
   
      k = mod( j - 1 , D ) ; cur=floor( (j-1)/D )  ; next = cur+1 ;
   
      slow_pp(:,j) = (1-k/D)*slow_pp (:,D*cur+1)  +  (k/D)*slow_pp (:,D*next+1) ;
      
      slow_pm(:,j) = (1-k/D)*slow_pm (:,D*cur+1)  +  (k/D)*slow_pm (:,D*next+1) ;
      
      msh(:,j,5)   = (1-k/D)*   msh(:,D*cur+1,5)  +  (k/D)*     msh(:,D*next+1,5) ;
      
     slow_var(:,j) = (1-k/D)*slow_var(:,D*cur+1)  +  (k/D)*slow_var(:,D*next+1) ;
     
end

msh(:,:,3) = slow_pp(:,:).*msh(:,:,1);

msh(:,:,4) = slow_pm(:,:).*msh(:,:,2);

msh(:,:,6) = slow_var(:,:).*msh(:,:,1).*conj(msh(:,:,2));


end
